@@ -8,7 +8,7 @@ import os
 import os.path as osp
 
 from dataclasses import dataclass, field, fields
-from functools import cached_property
+from functools import lru_cache
 from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, Union
 from zipfile import ZipFile
 
@@ -254,7 +254,7 @@ METADATA_KEYS = (
 
 class Clotho(Dataset[Dict[str, Any]]):
     r"""
-    Unofficial Clotho pytorch dataset.
+    Unofficial Clotho PyTorch dataset.
     Subsets available are 'train', 'val', 'eval', 'test' and 'analysis'.
 
     Audio are waveform sounds of 15 to 30 seconds, sampled at 44100 Hz.
@@ -336,27 +336,28 @@ class Clotho(Dataset[Dict[str, Any]]):
         """
         if version not in Clotho.VERSIONS:
             raise ValueError(
-                f"Invalid Clotho argument {version=}. Must be one of {Clotho.VERSIONS}."
+                f"Invalid Clotho argument version={version}. Must be one of {Clotho.VERSIONS}."
             )
 
-        if subset not in CLOTHO_LINKS[version].keys():
+        subsets = tuple(CLOTHO_LINKS[version].keys())
+        if subset not in subsets:
             raise ValueError(
-                f"Invalid Clotho argument {subset=} for {version=}. Must be one of {tuple(CLOTHO_LINKS[version].keys())}."
+                f"Invalid Clotho argument subset={subset} for version={version}. Must be one of {subsets}."
             )
 
         super().__init__()
-        self.__root = root
-        self.__subset = subset
-        self.__download = download
-        self.__transform = transform
-        self.__flat_captions = flat_captions
-        self.__version = version
-        self.__verbose = verbose
+        self._root = root
+        self._subset = subset
+        self._download = download
+        self._transform = transform
+        self._flat_captions = flat_captions
+        self._version = version
+        self._verbose = verbose
 
-        self.__all_items = {}
-        self.__loaded = False
+        self._all_items = {}
+        self._loaded = False
 
-        if self.__download:
+        if self._download:
             self.__prepare_data()
         self.__load_data()
 
@@ -367,7 +368,7 @@ class Clotho(Dataset[Dict[str, Any]]):
         return [
             field_.name
             for field_ in fields(ClothoItem)
-            if field_.name in self.__all_items or field_.name not in METADATA_KEYS
+            if field_.name in self._all_items or field_.name not in METADATA_KEYS
         ]
 
     @property
@@ -375,8 +376,8 @@ class Clotho(Dataset[Dict[str, Any]]):
         """Return the global dataset info."""
         return {
             "dataset": "clotho",
-            "subset": self.__subset,
-            "version": self.__version,
+            "subset": self._subset,
+            "version": self._version,
         }
 
     @property
@@ -404,13 +405,18 @@ class Clotho(Dataset[Dict[str, Any]]):
         if not isinstance(column, str) and isinstance(column, Iterable):
             return {column_i: self.at(idx, column_i) for column_i in column}
 
-        if isinstance(idx, (int, slice)) and column in self.__all_items.keys():
-            return self.__all_items[column][idx]
+        if isinstance(idx, (int, slice)) and column in self._all_items.keys():
+            return self._all_items[column][idx]
 
         if isinstance(idx, slice):
             idx = range(len(self))[idx]
 
         if isinstance(idx, Iterable):
+            idx = list(idx)
+            if not all(isinstance(idx_i, int) for idx_i in idx):
+                raise TypeError(
+                    f"Invalid input type for idx={idx}. (expected Iterable[int], not Iterable[{idx.__class__.__name__}])"
+                )
             return [self.at(idx_i, column) for idx_i in idx]
 
         if column == "audio":
@@ -420,11 +426,11 @@ class Clotho(Dataset[Dict[str, Any]]):
             # Sanity check
             if audio.nelement() == 0:
                 raise RuntimeError(
-                    f"Invalid audio number of elements in {fpath}. (expected {audio.nelement()=} > 0)"
+                    f"Invalid audio number of elements in {fpath}. (expected audio.nelement()={audio.nelement()} > 0)"
                 )
             if sr != self.SAMPLE_RATE:
                 raise RuntimeError(
-                    f"Invalid sample rate in {fpath}. (expected {self.SAMPLE_RATE} but found {sr=})"
+                    f"Invalid sample rate in {fpath}. (expected {self.SAMPLE_RATE} but found sr={sr})"
                 )
             return audio
 
@@ -457,23 +463,23 @@ class Clotho(Dataset[Dict[str, Any]]):
             return audio_metadata.sample_rate
 
         elif column == "subset":
-            return self.__subset
+            return self._subset
 
         else:
             raise ValueError(
-                f"Invalid argument {column=} at {idx=}. (expected one of {tuple(self.column_names)})"
+                f"Invalid argument column={column} at idx={idx}. (expected one of {tuple(self.column_names)})"
             )
 
     def is_loaded(self) -> bool:
         """Returns True if the dataset is loaded."""
-        return self.__loaded
+        return self._loaded
 
     def set_transform(
         self,
         transform: Optional[Callable[[Dict[str, Any]], Any]],
     ) -> None:
         """Set the transform applied to each row."""
-        self.__transform = transform
+        self._transform = transform
 
     # Magic methods
     def __getitem__(
@@ -490,52 +496,57 @@ class Clotho(Dataset[Dict[str, Any]]):
             column = None
 
         item = self.at(idx, column)
-        if self.__transform is not None:
-            item = self.__transform(item)
+        if self._transform is not None:
+            item = self._transform(item)
         return item
 
     def __len__(self) -> int:
         """
         :return: The number of items in the dataset.
         """
-        return len(self.__all_items["captions"])
+        return len(self._all_items["captions"])
 
     def __repr__(self) -> str:
-        return f"Clotho(size={len(self)}, subset={self.__subset}, num_columns={len(self.column_names)}, version={self.__version})"
+        return f"Clotho(size={len(self)}, subset={self._subset}, num_columns={len(self.column_names)}, version={self._version})"
 
     # Private methods
-    @cached_property
+    @property
+    @lru_cache()
     def __dpath_archives(self) -> str:
         return osp.join(self.__dpath_data, "archives")
 
-    @cached_property
+    @property
+    @lru_cache()
     def __dpath_audio(self) -> str:
         return osp.join(self.__dpath_data, "clotho_audio_files")
 
-    @cached_property
+    @property
+    @lru_cache()
     def __dpath_audio_subset(self) -> str:
         return osp.join(
             self.__dpath_data,
             "clotho_audio_files",
-            CLOTHO_AUDIO_DNAMES[self.__subset],
+            CLOTHO_AUDIO_DNAMES[self._subset],
         )
 
-    @cached_property
+    @property
+    @lru_cache()
     def __dpath_csv(self) -> str:
         return osp.join(self.__dpath_data, "clotho_csv_files")
 
-    @cached_property
+    @property
+    @lru_cache()
     def __dpath_data(self) -> str:
-        return osp.join(self.__root, f"CLOTHO_{self.__version}")
+        return osp.join(self._root, f"CLOTHO_{self._version}")
 
     def __is_prepared(self) -> bool:
         if not all(map(osp.isdir, (self.__dpath_audio_subset, self.__dpath_csv))):
             return False
 
-        if Clotho.CAPTIONS_PER_AUDIO[self.__subset] == 0:
+        if Clotho.CAPTIONS_PER_AUDIO[self._subset] == 0:
             return True
 
-        links = CLOTHO_LINKS[self.__version][self.__subset]
+        links = CLOTHO_LINKS[self._version][self._subset]
         captions_fname = links["captions"]["fname"]
         captions_fpath = osp.join(self.__dpath_csv, captions_fname)
         with open(captions_fpath, "r") as file:
@@ -546,11 +557,11 @@ class Clotho(Dataset[Dict[str, Any]]):
     def __load_data(self) -> None:
         if not self.__is_prepared():
             raise RuntimeError(
-                f"Cannot load data: clotho_{self.__subset} is not prepared in data root={self.__root}. Please use download=True in dataset constructor."
+                f"Cannot load data: clotho_{self._subset} is not prepared in data root={self._root}. Please use download=True in dataset constructor."
             )
 
         # Read fpath of .wav audio files
-        links = CLOTHO_LINKS[self.__version][self.__subset]
+        links = CLOTHO_LINKS[self._version][self._subset]
         dpath_audio_subset = self.__dpath_audio_subset
 
         if not osp.isdir(dpath_audio_subset):
@@ -573,13 +584,13 @@ class Clotho(Dataset[Dict[str, Any]]):
             metadata_fpath = osp.join(self.__dpath_csv, metadata_fname)
 
             # Keys: file_name, keywords, sound_id, sound_link, start_end_samples, manufacturer, license
-            if self.__version in ("v2", "v2.1"):
+            if self._version in ("v2", "v2.1"):
                 encoding = "ISO-8859-1"
             else:
                 encoding = None
 
             with open(metadata_fpath, "r", encoding=encoding) as file:
-                delimiter = ";" if self.__subset == "test" else ","
+                delimiter = ";" if self._subset == "test" else ","
                 reader = csv.DictReader(file, delimiter=delimiter)
                 metadata = list(reader)
         else:
@@ -619,7 +630,7 @@ class Clotho(Dataset[Dict[str, Any]]):
             fname = line["file_name"]
             if fname not in fname_to_idx:
                 raise KeyError(
-                    f"Cannot find metadata {fname=} in captions file. (subset={self.__subset})"
+                    f"Cannot find metadata fname={fname} in captions file. (subset={self._subset})"
                 )
             idx = fname_to_idx[fname]
             for key in subset_metadata_keys:
@@ -630,7 +641,8 @@ class Clotho(Dataset[Dict[str, Any]]):
         all_items = {
             "fname": fnames_lst,
             "captions": all_captions_lst,
-        } | all_metadata_dic
+        }
+        all_items.update(all_metadata_dic)
 
         if "keywords" in all_items:
             # Split keywords into list[str]
@@ -639,7 +651,7 @@ class Clotho(Dataset[Dict[str, Any]]):
                 for keywords in all_items["keywords"]
             ]
 
-        if self.__flat_captions and self.CAPTIONS_PER_AUDIO[self.__subset] > 1:
+        if self._flat_captions and self.CAPTIONS_PER_AUDIO[self._subset] > 1:
             all_infos_unfolded = {key: [] for key in all_items.keys()}
 
             for i, captions in enumerate(all_items["captions"]):
@@ -650,24 +662,24 @@ class Clotho(Dataset[Dict[str, Any]]):
 
             all_items = all_infos_unfolded
 
-        self.__all_items = all_items
-        self.__loaded = True
+        self._all_items = all_items
+        self._loaded = True
 
-        if self.__verbose >= 1:
+        if self._verbose >= 1:
             logger.info(f"{repr(self)} has been loaded. (len={len(self)})")
 
     def __prepare_data(self) -> None:
-        if not osp.isdir(self.__root):
-            raise RuntimeError(f"Cannot find root directory '{self.__root}'.")
+        if not osp.isdir(self._root):
+            raise RuntimeError(f"Cannot find root directory '{self._root}'.")
 
         os.makedirs(self.__dpath_archives, exist_ok=True)
         os.makedirs(self.__dpath_audio, exist_ok=True)
         os.makedirs(self.__dpath_csv, exist_ok=True)
 
-        if self.__verbose >= 1:
-            logger.info(f"Start to download files for clotho_{self.__subset}...")
+        if self._verbose >= 1:
+            logger.info(f"Start to download files for clotho_{self._subset}...")
 
-        links = copy.deepcopy(CLOTHO_LINKS[self.__version][self.__subset])
+        links = copy.deepcopy(CLOTHO_LINKS[self._version][self._subset])
         extensions = ("7z", "csv", "zip")
 
         # Download csv and 7z files
@@ -685,22 +697,22 @@ class Clotho(Dataset[Dict[str, Any]]):
                 dpath = self.__dpath_csv
             else:
                 raise RuntimeError(
-                    f"Found invalid {extension=}. Must be one of {extensions}."
+                    f"Found invalid extension={extension}. Must be one of {extensions}."
                 )
 
             fpath = osp.join(dpath, fname)
             if not osp.isfile(fpath) or self.FORCE_PREPARE_DATA:
-                if self.__verbose >= 1:
-                    logger.info(f"Download and check file '{fname}' from {url=}...")
+                if self._verbose >= 1:
+                    logger.info(f"Download and check file '{fname}' from url={url}...")
 
                 download_url_to_file(
                     url,
                     fpath,
-                    progress=self.__verbose >= 1,
+                    progress=self._verbose >= 1,
                 )
 
-            elif self.__verbose >= 1:
-                logger.info(f"File {fname=} is already extracted.")
+            elif self._verbose >= 1:
+                logger.info(f"File fname={fname} is already extracted.")
 
             with open(fpath, "rb") as file:
                 valid = validate_file(file, hash_value, hash_type="md5")
@@ -715,8 +727,8 @@ class Clotho(Dataset[Dict[str, Any]]):
             if extension in ("7z", "zip"):
                 fpath = osp.join(self.__dpath_archives, fname)
 
-                if self.__verbose >= 1:
-                    logger.info(f"Extract archive file {fname=}...")
+                if self._verbose >= 1:
+                    logger.info(f"Extract archive file fname={fname}...")
 
                 if extension == "7z":
                     archive_file = SevenZipFile(fpath)
@@ -735,7 +747,7 @@ class Clotho(Dataset[Dict[str, Any]]):
                 compressed_fnames = [
                     fname
                     for fname in compressed_fnames
-                    if fname not in ("", CLOTHO_AUDIO_DNAMES[self.__subset])
+                    if fname not in ("", CLOTHO_AUDIO_DNAMES[self._subset])
                 ]
                 extracted_fnames = (
                     os.listdir(self.__dpath_audio_subset)
@@ -760,7 +772,7 @@ class Clotho(Dataset[Dict[str, Any]]):
 
             else:
                 logger.error(
-                    f"Found unexpected {extension=} for downloaded file '{fname}'. Expected one of {extensions}."
+                    f"Found unexpected extension={extension} for downloaded file '{fname}'. Expected one of {extensions}."
                 )
 
         if self.CLEAN_ARCHIVES:
@@ -770,7 +782,7 @@ class Clotho(Dataset[Dict[str, Any]]):
 
                 if extension in ("7z", "zip"):
                     fpath = osp.join(self.__dpath_audio, fname)
-                    if self.__verbose >= 1:
+                    if self._verbose >= 1:
                         logger.info(f"Removing archive file {osp.basename(fpath)}...")
                     os.remove(fpath)
 
